@@ -386,7 +386,7 @@ print.bal.tab.subclass <- function(x, disp.m.threshold = "as.is", disp.v.thresho
     }
     if (!is.null(maximbal.r.subclass)) {
         cat("Variable with the greatest treatment correlation across subclasses:\n")
-        print.data.frame(round_df(maximbal.r.subclass, digits), row.names = FALSE)
+        print.data.frame(round_df(maximbal.r.subclass, digits), row.names = TRUE)
         cat("\n")
     }
     if (!is.null(baltal.m.subclass)) {
@@ -396,7 +396,7 @@ print.bal.tab.subclass <- function(x, disp.m.threshold = "as.is", disp.v.thresho
     }
     if (!is.null(maximbal.m.subclass)) {
         cat("Variable with the greatest mean difference across subclasses:\n")
-        print.data.frame(round_df(maximbal.m.subclass, digits), row.names = FALSE)
+        print.data.frame(round_df(maximbal.m.subclass, digits), row.names = TRUE)
         cat("\n")
     }
     if (!is.null(baltal.v.subclass)) {
@@ -406,7 +406,7 @@ print.bal.tab.subclass <- function(x, disp.m.threshold = "as.is", disp.v.thresho
     }
     if (!is.null(maximbal.v.subclass)) {
         cat("Variable with the greatest variance ratios across subclasses:\n")
-        print.data.frame(round_df(maximbal.v.subclass, digits), row.names = FALSE)
+        print.data.frame(round_df(maximbal.v.subclass, digits), row.names = TRUE)
         cat("\n")
     }
     if (!is.null(baltal.ks.subclass)) {
@@ -416,7 +416,7 @@ print.bal.tab.subclass <- function(x, disp.m.threshold = "as.is", disp.v.thresho
     }
     if (!is.null(maximbal.ks.subclass)) {
         cat("Variable with the greatest KS statistc across subclasses:\n")
-        print.data.frame(round_df(maximbal.ks.subclass, digits), row.names = FALSE)
+        print.data.frame(round_df(maximbal.ks.subclass, digits), row.names = TRUE)
         cat("\n")
     }
     
@@ -1443,7 +1443,8 @@ print.bal.tab.multi <- function(x, disp.m.threshold = "as.is", disp.v.threshold
     
     if (length(disp.treat.pairs) > 0) {
         headings <- setNames(character(length(disp.treat.pairs)), disp.treat.pairs)
-        cat("Balance by treatment pair:\n")
+        if (p.ops$pairwise) cat("Balance by treatment pair:\n")
+        else cat("Balance by treatment group:\n")
         for (i in disp.treat.pairs) {
             headings[i] <- paste0("\n - - - ", m.balance[[i]]$print.options$treat.names[1]," (0) vs. ",
                                   m.balance[[i]]$print.options$treat.names[2]," (1) - - - \n")
