@@ -343,9 +343,6 @@ get.w.mnps <- function(mnps, stop.method = NULL, s.weights = FALSE, ...) {
                 message(paste0("Warning: stop.method should be ", word.list(mnps$stopMethods, and.or = "or", quotes = TRUE), ".\nUsing all available stop methods instead."))
                 rule1 <- mnps$stopMethods
             }
-            # rule1 <- tryCatch(match.arg(tolower(stop.method), tolower(names(ps$w)), several.ok = TRUE),
-            #                   error = function(cond) {message(paste0("Warning: stop.method should be ", word.list(names(ps$w), and.or = "or", quotes = TRUE), ".\nUsing all available stop methods instead."));
-            #                       return(names(ps$w))})
         }
         else if (is.numeric(stop.method) && any(stop.method %in% seq_along(mnps$stopMethods))) {
             if (any(!stop.method %in% seq_along(mnps$stopMethods))) {
@@ -508,7 +505,7 @@ get.w.ebalance <- function(e, treat, ...) {
     if (missing(treat)) stop("treat must be specified.", call. = FALSE)
     
     weights <- rep(1, length(treat))
-    
+
     if (length(e$w) != sum(treat == 0)) {
         stop("There are more control units in treat than weights in the ebalance object.", call. = FALSE)
     }
@@ -554,7 +551,7 @@ word.list <- function(word.list = NULL, and.or = c("and", "or"), is.are = FALSE,
                 out <- paste(word.list, collapse = paste0(" ", and.or," "))
             }
             else {
-                out <- paste(paste(word.list[seq_len(L-1)], collapse = ", "), 
+                out <- paste(paste(word.list[seq_len(L-1)], collapse = ", "),
                              word.list[L], sep = paste0(", ", and.or," "))
                 
             }
@@ -569,13 +566,23 @@ word.list <- function(word.list = NULL, and.or = c("and", "or"), is.are = FALSE,
 expand.grid_string <- function(..., collapse = "") {
     return(apply(expand.grid(...), 1, paste, collapse = collapse))
 }
-nunique <- function(x, nmax = NA) {
-    if (is.factor(x)) return(nlevels(x))
-    else return(length(unique(x, nmax = nmax)))
+nunique <- function(x, nmax = NA, na.rm = TRUE) {
+    if (length(x) == 0) return(0)
+    else {
+        if (na.rm) x <- x[!is.na(x)]
+        if (is.factor(x)) return(nlevels(x))
+        else return(length(unique(x, nmax = nmax)))
+    }
+    
 }
-nunique.gt <- function(x, n) {
-    if (length(x) < 2000) nunique(x) > n
-    else tryCatch(nunique(x, nmax = n) > n, error = function(e) TRUE)
+nunique.gt <- function(x, n, na.rm = TRUE) {
+    if (n < 0) stop("n must be non-negative.", call. = FALSE)
+    if (length(x) == 0) FALSE
+    else {
+        if (na.rm) x <- x[!is.na(x)]
+        if (length(x) < 2000) nunique(x) > n
+        else tryCatch(nunique(x, nmax = n) > n, error = function(e) TRUE)        
+    }
 }
 is.formula <- function(f, sides = NULL) {
     res <- is.name(f[[1]])  && deparse(f[[1]]) %in% c( '~', '!') &&
@@ -584,20 +591,4 @@ is.formula <- function(f, sides = NULL) {
         res <- res && length(f) == sides + 1
     }
     return(res)
-}
-
-#Under construction
-inxnoty <- function(x, y) {
-    #Creates a list or data frame of names in x that are not in y.
-    #Useful for subsetting data sets into two groups of variables.
-    
-    if (!(is.character(x) || is.data.frame(x) || is.matrix(x)) || !(is.character(y) || is.data.frame(y) || is.matrix(y))) {
-        stop("Inputs to x and y must be either strings containing variable names or data frames or matrices with named columns.", call. = FALSE)
-    }
-    if (is.character(x)) X <- x else X <- colnames(x)
-    if (is.character(y)) Y <- y else Y <- colnames(y)
-    
-    if(is.character(x)) out <- x[is.na(match(X, Y))]
-    else out <- x[, is.na(match(X, Y))]
-    return(out)
-}
+}
\ No newline at end of file
